@@ -39,6 +39,9 @@ const matlab_saveobj_ret_types = String[
     "timetable"
 ]
 
+# Warning message for unknown regions
+const warn_msg = "Unknown metadata found in MCOS subsystem. Please raise an issue to the maintainers to help improve support."
+
 mutable struct Subsystem
     load_object_cache::Dict{UInt32,MatlabOpaque}
     save_object_cache::IdDict{MatlabOpaque,UInt32}
@@ -60,7 +63,7 @@ mutable struct Subsystem
 
     # automatic MatlabOpaque conversion
     convert_opaque::Bool
-    table_type::Type 
+    table_type::Type
 
     # Counters for saving
     saveobj_counter::UInt32
@@ -179,6 +182,34 @@ function load_mcos_regions!(
     end
 end
 
+function check_unknown_regions(subsys::Subsystem)
+    # Warn about unsupported regions
+    # We don't know what they contain yet
+    # Users can raise issues if they see this warning to help improve support
+
+    if length(subsys._u6_metadata) > 0
+        # This region is supposedly empty
+        @warn warn_msg
+    end
+
+    if any(subsys._u7_metadata .!= 0)
+        # This is an 8 byte region, all zeros
+        @warn warn_msg
+    end
+
+    if subsys._c3 === nothing
+        return
+    else
+        for i=1:length(subsys._c3)
+            # This region is a cell array of empty structs
+            if length(subsys._c3[i]) > 0
+                @warn warn_msg
+                break
+            end
+        end
+    end
+end
+
 function get_region(
     fwrap_metadata::Vector{UInt8}, region_offsets::AbstractVector{UInt32}, region::Integer
 )
@@ -186,47 +217,59 @@ function get_region(
 end
 
 function load_subsys!(subsys::Subsystem, subsystem_data::Dict{String,Any}, swap_bytes::Bool)
-    subsys.handle_data = get(subsystem_data, "handle", nothing)
-    subsys.java_data = get(subsystem_data, "java", nothing)
-    mcos_data = get(subsystem_data, "MCOS", nothing)
-    if mcos_data === nothing
-        return nothing
+    try
+        subsys.handle_data = get(subsystem_data, "handle", nothing)
+        subsys.java_data = get(subsystem_data, "java", nothing)
+        mcos_data = get(subsystem_data, "MCOS", nothing)
+        if mcos_data === nothing
+            return nothing
+        end
+
+        if mcos_data isa Tuple
+            # Backward compatibility with MAT_v5
+            mcos_data = mcos_data[2]
+        end
+        fwrap_metadata::Vector{UInt8} = vec(mcos_data[1, 1])
+
+        version = swapped_reinterpret(fwrap_metadata[1:4], swap_bytes)[1]
+        if version <= 1 || version > FWRAP_VERSION
+            error("Cannot read subsystem: Unsupported FileWrapper version: $version")
+        end
+
+        subsys.num_names = swapped_reinterpret(fwrap_metadata[5:8], swap_bytes)[1]
+        load_mcos_names!(subsys, fwrap_metadata)
+
+        load_mcos_regions!(subsys, fwrap_metadata, swap_bytes)
+
+        if version == 2
+            subsys.prop_vals_saved = mcos_data[3:(end - 1), 1]
+        elseif version == 3
+            subsys.prop_vals_saved = mcos_data[3:(end - 2), 1]
+            subsys.mcos_class_alias_metadata = mcos_data[end - 1, 1]
+        else
+            subsys.prop_vals_saved = mcos_data[3:(end - 3), 1]
+            subsys._c3 = mcos_data[end - 2, 1]
+        end
+
+        subsys.prop_vals_defaults = mcos_data[end, 1]
+        check_unknown_regions(subsys)
+        return subsys
+    catch
+        @warn "Failed to load MAT-file subsystem data. Opaque objects will be skipped. Error: $(catch_backtrace())"
+        return subsys
     end
-
-    if mcos_data isa Tuple
-        # Backward compatibility with MAT_v5
-        mcos_data = mcos_data[2]
-    end
-    fwrap_metadata::Vector{UInt8} = vec(mcos_data[1, 1])
-
-    version = swapped_reinterpret(fwrap_metadata[1:4], swap_bytes)[1]
-    if version <= 1 || version > FWRAP_VERSION
-        error("Cannot read subsystem: Unsupported FileWrapper version: $version")
-    end
-
-    subsys.num_names = swapped_reinterpret(fwrap_metadata[5:8], swap_bytes)[1]
-    load_mcos_names!(subsys, fwrap_metadata)
-
-    load_mcos_regions!(subsys, fwrap_metadata, swap_bytes)
-
-    if version == 2
-        subsys.prop_vals_saved = mcos_data[3:(end - 1), 1]
-    elseif version == 3
-        subsys.prop_vals_saved = mcos_data[3:(end - 2), 1]
-        subsys.mcos_class_alias_metadata = mcos_data[end - 1, 1]
-    else
-        subsys.prop_vals_saved = mcos_data[3:(end - 3), 1]
-        subsys._c3 = mcos_data[end - 2, 1]
-    end
-
-    subsys.prop_vals_defaults = mcos_data[end, 1]
-    return subsys
 end
 
 function get_classname(subsys::Subsystem, class_id::UInt32)
     namespace_idx = subsys.class_id_metadata[class_id * 4 + 1]
     classname_idx = subsys.class_id_metadata[class_id * 4 + 2]
 
+    if subsys.class_id_metadata[class_id * 4 + 3] != 0 ||
+       subsys.class_id_metadata[class_id * 4 + 4] != 0
+       # Unknown metadata
+       @warn warn_msg
+    end
+
     namespace = if namespace_idx == 0
         ""
     else
@@ -325,10 +368,13 @@ end
 
 function get_dynamic_properties(subsys::Subsystem, dep_id::UInt32)
 
-    if length(subsys.dynprop_metadata) == 0
+    if dep_id == UInt32(0)
         return Dict{String,Any}()
     end
 
+    # dynprops metadata is stored in format
+    # [(nprops, prop_obj_id_1, ..., prop_obj_id_n), ...] for each dep_id
+
     offset = 1
     while dep_id > 0
         nprops = subsys.dynprop_metadata[offset]
@@ -341,7 +387,7 @@ function get_dynamic_properties(subsys::Subsystem, dep_id::UInt32)
     offset += 1
     dyn_prop_obj_ids = subsys.dynprop_metadata[offset:(offset + ndynprops - 1)]
 
-    if dyn_prop_obj_ids == UInt32[]
+    if length(dyn_prop_obj_ids) == 0
         return Dict{String,Any}()
     end
     dyn_prop_map = Dict{String,Any}()
@@ -361,7 +407,12 @@ function get_properties(subsys::Subsystem, object_id::UInt32)
         return Dict{String,Any}()
     end
 
-    class_id, _, _, saveobj_id, normobj_id, _ = get_object_metadata(subsys, object_id)
+    class_id, _x1, _x2, saveobj_id, normobj_id, dep_id = get_object_metadata(subsys, object_id)
+    if _x1 != 0 || _x2 != 0
+        # Unknown metadata
+        @warn warn_msg
+    end
+
     if saveobj_id != 0
         saveobj_ret_type = true
         obj_type_id = saveobj_id
@@ -372,7 +423,7 @@ function get_properties(subsys::Subsystem, object_id::UInt32)
 
     defaults = get_default_properties(subsys, class_id)
     prop_map = merge(defaults, get_saved_properties(subsys, obj_type_id, saveobj_ret_type))
-    dyn_props = get_dynamic_properties(subsys, object_id)
+    dyn_props = get_dynamic_properties(subsys, dep_id)
     merge!(prop_map, dyn_props)
     return prop_map
 end
@@ -403,41 +454,53 @@ function load_mcos_object(metadata::Dict, type_name::String, subsys::Subsystem)
 end
 
 function load_mcos_object(metadata::Array{UInt32}, type_name::String, subsys::Subsystem)
-    if type_name != "MCOS"
-        @warn "Loading Type:$type_name is not implemented. Returning metadata."
-        return metadata
-    end
-
-    if metadata[1, 1] != MCOS_IDENTIFIER
-        @warn "MCOS object metadata is corrupted. Returning raw data."
-        return metadata
-    end
-
-    ndims = metadata[2, 1]
-    dims = metadata[3:(2 + ndims), 1]
-    nobjects = prod(dims)
-    object_ids = metadata[(3 + ndims):(2 + ndims + nobjects), 1]
-
-    class_id = metadata[end, 1]
-    classname = get_classname(subsys, class_id)
-
-    if nobjects == 1
-        oid = object_ids[1]
-        obj = get_object!(subsys, oid, classname)
-        if subsys.convert_opaque
-            return convert_opaque(obj; table=subsys.table_type)
-        else
-            return obj
+    try
+        if length(subsys.class_id_metadata) == 0
+            # Subsystem was not loaded properly
+            # No need to warn again
+            return metadata
         end
-    else
-        # no need to convert_opaque, matlab wraps object arrays in a single class normally
-        object_arr = Array{MatlabOpaque}(undef, convert(Vector{Int}, dims)...)
-        for i in 1:length(object_arr)
-            oid = object_ids[i]
+
+        if type_name != "MCOS"
+            @warn "Loading Type:$type_name is not implemented. Returning metadata."
+            return metadata
+        end
+
+        if metadata[1, 1] != MCOS_IDENTIFIER
+            @warn "MCOS object metadata is corrupted. Returning raw data."
+            return metadata
+        end
+
+        ndims = metadata[2, 1]
+        dims = metadata[3:(2 + ndims), 1]
+        nobjects = prod(dims)
+        object_ids = metadata[(3 + ndims):(2 + ndims + nobjects), 1]
+
+        class_id = metadata[end, 1]
+        classname = get_classname(subsys, class_id)
+
+        if nobjects == 1
+            oid = object_ids[1]
             obj = get_object!(subsys, oid, classname)
-            object_arr[i] = obj
+            if subsys.convert_opaque
+                return convert_opaque(obj; table=subsys.table_type)
+            else
+                return obj
+            end
+        else
+            # no need to convert_opaque, matlab wraps object arrays in a single class normally
+            object_arr = Array{MatlabOpaque}(undef, convert(Vector{Int}, dims)...)
+            for i in 1:length(object_arr)
+                oid = object_ids[i]
+                obj = get_object!(subsys, oid, classname)
+                object_arr[i] = obj
+            end
+            return object_arr
         end
-        return object_arr
+
+    catch e
+        @warn "Failed to load MCOS object. Returning raw metadata. Error: $e"
+        return metadata
     end
 end
 
